@@ -276,7 +276,9 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 	if(cu && cu->countUnits() == 0) return mstruct;
 	int exp1, exp2;
 	bool b_ratio = cu && cu->countUnits() == 2 && cu->get(1, &exp1)->baseUnit() == cu->get(2, &exp2)->baseUnit() && exp1 == -exp2;
-	if(!b_ratio && !mstruct.containsType(STRUCT_UNIT, true)) return mstruct;
+	if(!b_ratio && to_unit->baseUnit() != getRadUnit() && !mstruct.containsType(STRUCT_UNIT, true)) {
+		return mstruct;
+	}
 	MathStructure mstruct_new(mstruct);
 	size_t n_messages = messages.size();
 	if(to_unit->hasNonlinearRelationTo(to_unit->baseUnit()) && to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
@@ -310,7 +312,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 					if(!b_changed && !mstruct_new.equals(mstruct[i], true, true)) b_changed = true;
 				}
 			}
-			if(b_changed) {
+			if(b_changed && !b_ratio) {
 				mstruct_new.childrenUpdated();
 				EvaluationOptions eo2 = eo;
 				//eo2.calculate_functions = false;
@@ -326,7 +328,25 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 		eo2.keep_prefixes = true;
 		bool b = false;
 		if(eo.approximation == APPROXIMATION_EXACT) eo2.approximation = APPROXIMATION_TRY_EXACT;
-		if(mstruct_new.convert(to_unit, true, NULL, false, eo2, eo.keep_prefixes ? decimal_null_prefix : NULL) || always_convert || b_ratio) {
+		if(mstruct_new.convert(to_unit, true, NULL, false, eo2, eo.keep_prefixes ? decimal_null_prefix : NULL)) {
+			b = true;
+		} else if(b_ratio) {
+			int exp;
+			if(cu->get(1, &exp)->baseUnit()->referenceName() == "m") {
+				Unit *u = NULL;
+				if(exp == 1 || exp == -1) u = getRadUnit();
+				else if(exp == 2 || exp == -2) u = getActiveUnit("sr");
+				if(u) {
+					mstruct_new.convert(u, true, NULL, false, eo2, NULL);
+					if(mstruct_new.contains(u, false, false, false)) mstruct_new.replace(u, m_one, false, false);
+				}
+			}
+			b = true;
+		} else if(to_unit->baseUnit() == getRadUnit() && mstruct_new.contains(to_unit, true) < 1) {
+			mstruct_new.multiply(getRadUnit(), true);
+			if(to_unit->baseExponent() != 1) mstruct_new.last().raise(to_unit->baseExponent());
+			b = mstruct_new.convert(to_unit, true, NULL, false, eo2, eo.keep_prefixes ? decimal_null_prefix : NULL) || always_convert;
+		} else if(always_convert) {
 			b = true;
 		} else {
 			CompositeUnit *cu2 = cu;
@@ -448,6 +468,14 @@ MathStructure Calculator::convertToBaseUnits(const MathStructure &mstruct, const
 		mstruct_new.eval(eo2);
 		cleanMessages(mstruct, n_messages + 1);
 	}
+	if(mstruct_new.contains(getRadUnit(), false, false, false)) {
+		Unit *u = getActiveUnit("m");
+		if(u) {
+			MathStructure m_p_m(u);
+			m_p_m.divide(u);
+			mstruct_new.replace(getRadUnit(), m_p_m, false, true);
+		}
+	}
 	return mstruct_new;
 }
 Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
@@ -711,7 +739,11 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 									cu_mstruct.raise(b_exp);
 								}
 							}
-							cu_mstruct = convertToBaseUnits(cu_mstruct);
+							if(cu_mstruct.convertToBaseUnits(true, NULL, true)) {
+								beginTemporaryStopMessages();
+								cu_mstruct.eval();
+								endTemporaryStopMessages();
+							}
 							if(cu_mstruct.isMultiplication()) {
 								for(size_t i2 = 1; i2 <= cu_mstruct.countChildren(); i2++) {
 									bu = NULL;
@@ -792,7 +824,11 @@ Unit *Calculator::getOptimalUnit(Unit *u, bool allow_only_div, bool convert_to_l
 				MathStructure cu_mstruct = ((CompositeUnit*) u)->generateMathStructure();
 				if(minus) cu_mstruct *= best_u;
 				else cu_mstruct /= best_u;
-				cu_mstruct = convertToBaseUnits(cu_mstruct);
+				if(cu_mstruct.convertToBaseUnits(true, NULL, true)) {
+					beginTemporaryStopMessages();
+					cu_mstruct.eval();
+					endTemporaryStopMessages();
+				}
 				CompositeUnit *cu2 = new CompositeUnit("", "temporary_composite_convert_to_optimal_unit");
 				bool b = false;
 				for(size_t i = 1; i <= cu_mstruct.countChildren(); i++) {
@@ -1199,7 +1235,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct_to_convert, strin
 			}
 		}
 		current_stage = MESSAGE_STAGE_CONVERSION;
-		if(to_struct) to_struct->set(cu.generateMathStructure());
+		if(to_struct) to_struct->set(cu.generateMathStructure(true));
 		if(cu.countUnits() > 0) {
 			mstruct.set(convert(mstruct_to_convert, &cu, eo2, false, false));
 			b = true;
